/**********************************************************
 * S I M U L A T I O N  T E L E P H O N E  U43 EN 1 9 7 0 *
 * ********************************************************
 * 
 * Matériel utilisé : Microcontroleur Arduino & DFPlayer Mini
 *  
 * Auteur : Daniel Bourgeois 
 * Version : 1.00
 * Date : 11/02/2023
 * 
 */

#include "Arduino.h"

// ************ DECLARATION DU LECTEUR MP3 ************ 
#include "SoftwareSerial.h"                 // communication serie avec le lecteur mp3 DFPlayer
#include "DFRobotDFPlayerMini.h"            // bibliothèque du DFPlayer
SoftwareSerial mySoftwareSerial(10, 11);    // D10 RX Arduino avec TX DFPlayer, D11 TX Arduino avec RX DFPlayer + 1K Ohm 
DFRobotDFPlayerMini myDFPlayer;             // initialisation du DFPlayer
const unsigned short BROCHE_BUSY = 12;      // état du DFPlayer : 0=lecture encours 1=repos

// ************ DECLARATION DES ENTREES ARDUINO DU TELEPHONE ************ 
const unsigned short BROCHE_DECROCHAGE = 4; // câble bleu soudé sous la borne III du petit bornier carré. 0=raccroché 1=décroché  
const unsigned short BROCHE_ROTATION = 3;   // câble bleu du cadran rotatif. 0=en rotation 1=immobile
const unsigned short BROCHE_IMPULSIONS = 2; // câble rouge du cadran rotatif. 0=repos 1=impulsion

// ************ ETATS DU TELEPHONE ************
bool ETAT_RACCROCHAGE = 1;                  // position du combiné 0=décroché 1=raccroché
bool ETAT_RACCROCHAGE_PREC = 1;             // état précédent pour détecter les changements
bool ETAT_ROTATION = 1;                     // situation du cadran rotatif : 1=immobile 0=en rotation 
bool ETAT_ROTATION_PREC = 1;                // état précédent pour détecter les changements
bool ETAT_IMPULSION = 0;                    // réception des impulsions : 0=repos 1=impulsion encours
bool ETAT_IMPULSION_PREC = 0;               // état précédent pour détecter les changements

bool TONALITE = 0;                          // 0=pas de tonalité encours

// ************ ETATS D'AVANCEMENT DE LA COMMUNICATION *************
const unsigned short RACCROCHE = 0;         // Raccroché, attente décrochage
const unsigned short ATTNUM = 1;            // Décroché, attente de numérotation
const unsigned short NUMEROT = 2;           // Numérotation encours
const unsigned short ACHEMIN = 3;           // Numérotation terminée, acheminement
const unsigned short SONNE = 4;             // Sonnerie, attente du décrochage du destinataire
const unsigned short DEBCOM = 5;            // Début de communication
const unsigned short ENCOM = 6;           // Communication encours
const unsigned short ERREUR = 9;            // Erreur
unsigned short ETATEL = RACCROCHE;

// ************ ENREGISTREMENT DE LA NUMEROTATION ************ 
int CHIFFRE = 0;
int NBR_CHIFFRES = 0;
long NUMTEL = 0;
long FICHIER_CARTE = 0;
int DOSSIER_CARTE = 0;

// ************ TYPE D'APPEL *************
const unsigned short INDETERMINE = 0;
const unsigned short LOCAL = 1;
const unsigned short INTERURBAIN = 2;
const unsigned short INTERNATIONAL = 3;
const unsigned short SPECIAL = 4;
unsigned short TYPE_APPEL = INDETERMINE;

// ************ INDICATIFS TELEPHONIQUES INTERNATIONAUX *************
const unsigned short USA = 1;
const unsigned short RUSSIE = 7;
const unsigned short ESPAGNE = 34;
const unsigned short ITALIE = 39;
const unsigned short ROYAUME_UNI = 44;
const unsigned short ALLEMAGNE_RFA = 49;
const unsigned short ALLEMAGNE_RDA = 37;
const unsigned short CHINE = 86;
const unsigned short TAIWAN = 886;
const unsigned short JAPON = 81;
const unsigned short MAROC = 212;
const unsigned short BENIN = 229;
const unsigned short LA_REUNION = 262;
unsigned short INDICATIF_PAYS = 0;
unsigned short AREA_CODE = 0;

// ************ TEMPS D'ATTENTE *************
unsigned long TEMPO_DEBUT = 0;
unsigned long TEMPO_IMPULSION = 0;
const unsigned short T_INVIT_NUM_MAX = 10000; // Temps maximal d'attente avant le début de la numérotation (en ms)
const unsigned short T_NUMEROT_MAX = 5000;    // Temps maximal d'attente entre 2 chiffres (en ms)

boolean ETAT_LECTURE;                         // Etat de la lecture d'un fichier mp3 : 0=lecture ok 1=pas de lecture encours, donc problème 


void setup() {

  Serial.begin(9600);                         // Vitesse pour info sur la console

// ************ INITIALISATION DFPLAYER ************
  mySoftwareSerial.begin(9600);               // Vitesse du DFPlayer
    if (!myDFPlayer.begin(mySoftwareSerial,true,false)) { // les paramètres true, false suppriment le plop au démarrage
    Serial.println(F("Miniplayer pas ok"));
    while (true);
  }
  Serial.println("DFPlayer Mini ok");
  delay(2000);                                // Temps de démarrage du DFPlayer 
  myDFPlayer.volume(15);                      // Réglage du volume de 0 à 30

// ************ DECLARATION ENTREES ARDUINO ************
  pinMode(BROCHE_DECROCHAGE, INPUT_PULLUP);
  pinMode(BROCHE_ROTATION, INPUT_PULLUP);
  pinMode(BROCHE_IMPULSIONS, INPUT_PULLUP); // 66ms toutes les 100ms
  pinMode(BROCHE_BUSY, INPUT);   
}

void repos() {
  if (ETAT_RACCROCHAGE_PREC == 0) { // on vient de raccrocher
    CHIFFRE = 0;
    NUMTEL = 0;
    NBR_CHIFFRES = 0;
    TYPE_APPEL = INDETERMINE;
    INDICATIF_PAYS = 0;
    AREA_CODE = 0;
    FICHIER_CARTE = 0;
    DOSSIER_CARTE = 0;
    ETAT_RACCROCHAGE_PREC = 1;
    //    Serial.println("Raccrochage");
    myDFPlayer.stop();
    TONALITE = 0;
  } 
  else if (ETAT_RACCROCHAGE == 0) {
    ETATEL = ATTNUM;
    ETAT_RACCROCHAGE_PREC = 0;
    TEMPO_DEBUT = millis();
    //    Serial.print("Tempo debut  ");Serial.println(TEMPO_DEBUT);
     myDFPlayer.playFolder(1, 1); // Tonalité
    TONALITE = 1;
    //    Serial.println("Tonalité");
  }
  delay(100);
}

void attendre_numerotation() {
  if (ETAT_ROTATION == 0) {       //Cadran tourne
    ETATEL = NUMEROT;
  }
  else if (millis() - TEMPO_DEBUT > T_INVIT_NUM_MAX) {    // attente numérotation trop longue
    ETATEL = ERREUR;
    //    Serial.print("Occupation  ");Serial.println(millis()-TEMPO_DEBUT);
    myDFPlayer.playFolder(1, 2); // Occupation
    TONALITE = 1;
  }

  delay(10);
}

void numeroter() {
  if (ETAT_ROTATION == 0) { 
    
// ************ CADRAN TOURNE ***********************************************************************************  
      // Si début de la rotation du cadran, arréter la tonalité (prise de ligne, interurbain ou international)
    if (ETAT_ROTATION_PREC == 1 && TONALITE == 1) { 
      myDFPlayer.stop(); // Stop tonalité
      TONALITE = 0;
    }  

    ETAT_ROTATION_PREC = 0;

     // Si détéction d'une impulsion, initialiser le timer
    if (ETAT_IMPULSION == 1 && ETAT_IMPULSION_PREC == 0) { 
            //Serial.print("Time = "); Serial.println(millis());
            TEMPO_IMPULSION = millis();
    } 

    // Si détection d'une fin d'impulsion et si impulsion suffisamment longue, ajouter 1 au chiffre encours
    if (ETAT_IMPULSION == 0 && ETAT_IMPULSION_PREC == 1 && (millis()-TEMPO_IMPULSION >20) ){ 
        CHIFFRE++;
        //Serial.print("Time = "); Serial.println(millis()); 
        //Serial.print("Chiffre = "); Serial.println(CHIFFRE);  
    } 
    
    ETAT_IMPULSION_PREC = ETAT_IMPULSION;
    
  }  
  
  else { 
// ************ CADRAN IMMOBILE *************************************************************************************  
    if (ETAT_ROTATION_PREC == 0) { // le cadran vient juste de s'arrêter, ajouter le chiffre encours au numéro appelé
      NBR_CHIFFRES += 1;
      CHIFFRE %= 10; // modulo 10 pour que 10=0
      NUMTEL *= 10; // decalage 1 chiffre à gauche
      NUMTEL += CHIFFRE;
      CHIFFRE = 0;
      ETAT_IMPULSION = 0;
      ETAT_IMPULSION_PREC = 0;

// Détermination du type d'appel si on a les 2 premiers chiffres
      if (TYPE_APPEL == INDETERMINE && NBR_CHIFFRES == 2) { 
        switch (NUMTEL) {
          case 12: case 13: case 14: case 17: case 18:
            TYPE_APPEL = SPECIAL;
            break;
          case 16:
            TYPE_APPEL = INTERURBAIN;
            NUMTEL = 0; // On reprend la numérotation au début
            NBR_CHIFFRES = 0;
            myDFPlayer.playFolder(1, 7); // Tonalité interurbaine ou internationale
            TONALITE = 1;
            break;
          case 19:
            TYPE_APPEL = INTERNATIONAL;
            NUMTEL = 0; // On reprend la numérotation au début
            NBR_CHIFFRES = 0;
            myDFPlayer.playFolder(1, 7); // Tonalité interurbaine ou internationale
            TONALITE = 1;
            break;
          default :
            TYPE_APPEL = LOCAL;
            break;
        }
      }

      // Appel international: détermination du pays si suffisamment de chiffres 
      if (TYPE_APPEL == INTERNATIONAL && INDICATIF_PAYS == 0) {   
        if (  NUMTEL == USA ||
              NUMTEL == ESPAGNE || 
              NUMTEL == ROYAUME_UNI || 
              NUMTEL == ALLEMAGNE_RFA || 
              NUMTEL == ALLEMAGNE_RDA ||
              NUMTEL == CHINE ||
              NUMTEL == TAIWAN ||
              NUMTEL == JAPON ||
              NUMTEL == LA_REUNION ){
                INDICATIF_PAYS = NUMTEL; 
                NUMTEL = 0; // On reprend la numérotation au début
                NBR_CHIFFRES = 0;
              }
      }
      // Appel aux USA : détermination de l'area code si suffisamment de chiffres 
      if (TYPE_APPEL == INTERNATIONAL && INDICATIF_PAYS == USA && NBR_CHIFFRES == 3) {
        AREA_CODE = NUMTEL; 
        NUMTEL = 0; //On reprend la numérotation au début
      }

      // Appel en Allemagne : détermination de l'area code si suffisamment de chiffres 
      if (TYPE_APPEL == INTERNATIONAL && INDICATIF_PAYS == ALLEMAGNE_RFA && NBR_CHIFFRES == 4) {
        AREA_CODE = NUMTEL;
        NUMTEL = 0; //On reprend la numérotation au début
      }

      Serial.print("NumTel = "); Serial.println(NUMTEL);
      Serial.print("Nbr chiffres = "); Serial.println(NBR_CHIFFRES);
      Serial.print("Type appel = "); Serial.println(TYPE_APPEL);
      Serial.print("Indicatif pays = "); Serial.println(INDICATIF_PAYS);
      Serial.print("Area_code = "); Serial.println(AREA_CODE);      
      Serial.println(" ");

      ETAT_ROTATION_PREC = 1;
      TEMPO_DEBUT = millis();
    }
    else { // cadran immobile et chiffre deja enregistre

 // Numéro spécial ou appel local à 6 chiffres ou interurbain à 8 chiffres : numérotation terminée, acheminement    
      if (     (TYPE_APPEL == SPECIAL)
               || (TYPE_APPEL == LOCAL && NBR_CHIFFRES == 6)
               || (TYPE_APPEL == INTERURBAIN && NBR_CHIFFRES == 8) )
      { 
        ETATEL = ACHEMIN;
        TEMPO_DEBUT = millis();
        myDFPlayer.playFolder(1, 3); // Tonalité d'acheminement
        TONALITE = 1;
      }
      else {
        if (millis() - TEMPO_DEBUT > T_NUMEROT_MAX) {
          
 // Attente de numérotation trop longue         
          if (TYPE_APPEL == INTERNATIONAL && INDICATIF_PAYS !=0) { // fin de numérotation en international
            ETATEL = SONNE; // Retour d'appel, pas de tonalité d'acheminement en international
            TEMPO_DEBUT = millis();
            switch (INDICATIF_PAYS) {
              case USA: myDFPlayer.playFolder(1, 8); break; // retour d'appel USA
              case ROYAUME_UNI: myDFPlayer.playFolder(1, 9); break; // retour d'appel UK
              case CHINE: case TAIWAN: myDFPlayer.playFolder(1, 16); break; // retour d'appel Chine
              case JAPON: myDFPlayer.playFolder(1, 18); break; // retour d'appel Japon
              default : myDFPlayer.playFolder(1, 10); break; // retour d'appel Europe
            }
            TONALITE = 1;
          }
          else { // Attente anormale de numérotation
            ETATEL = ERREUR;
            myDFPlayer.playFolder(1, 2); // Tonalité d'occupation
          }

        }
      }
    }

  }
  delay(10);
}

void acheminer() {
  if (millis() - TEMPO_DEBUT > 2000) { // fin acheminement (sauf international)
    ETATEL = SONNE;
    TEMPO_DEBUT = millis();
    myDFPlayer.playFolder(1, 4); // retour d'appel France
    TONALITE = 1;
  }
  delay(200);
}

void sonner() {
  if (millis() - TEMPO_DEBUT > 5000) { // fin sonnerie
    ETATEL = DEBCOM;
  }
  delay(200);
}

void etablircom() {

if (TYPE_APPEL == SPECIAL){
  DOSSIER_CARTE = 2; 
  FICHIER_CARTE = NUMTEL;
}
else
  if ( (TYPE_APPEL == LOCAL)||(TYPE_APPEL == INTERURBAIN) ){
    switch (NUMTEL){
      case 322851: DOSSIER_CARTE = 3; FICHIER_CARTE = 1; break; // Jules
      case 542138: DOSSIER_CARTE = 3; FICHIER_CARTE = 2; break; // Noémie
      case 876325: DOSSIER_CARTE = 3; FICHIER_CARTE = 6; break; // Emma
      case 998400: DOSSIER_CARTE = 3; FICHIER_CARTE = 99; break; // Horloge parlante
      case 19283260: DOSSIER_CARTE = 3; FICHIER_CARTE = 3; break; // Fanny
      case 67512844: DOSSIER_CARTE = 3; FICHIER_CARTE = 4; break; // Katia (Montpellier)
      case 20590118: DOSSIER_CARTE = 3; FICHIER_CARTE = 5; break; // Josiane (Nord)
      default: DOSSIER_CARTE = 1; FICHIER_CARTE = 6; break;
    }
  }
    else {
      if (TYPE_APPEL == INTERNATIONAL) {
        Serial.println("Appel international");
        switch (INDICATIF_PAYS) {
          case USA: appel_usa(); Serial.println("Appel USA");break;
          case ROYAUME_UNI: appel_uk(); Serial.println("Appel UK");break;
          case ALLEMAGNE_RFA: appel_allemagne_rfa(); Serial.println("Appel Allemagne RFA");break;
          case ALLEMAGNE_RDA: appel_allemagne_rda(); Serial.println("Appel Allemagne RDA");break;
          case ESPAGNE: appel_espagne(); Serial.println("Appel Espagne");break;
          case CHINE: appel_chine(); Serial.println("Appel Chine");break;
          case TAIWAN: appel_taiwan(); Serial.println("Appel Taiwan");break;
          case JAPON: appel_japon(); Serial.println("Appel Japon");break;
          case LA_REUNION: appel_reunion(); Serial.println("Appel Réunion");break;
        }
      }

    }
 
  myDFPlayer.playFolder(DOSSIER_CARTE, FICHIER_CARTE);
  delay(500);
  ETAT_LECTURE = digitalRead(BROCHE_BUSY); // HIGH = not playing
  if (ETAT_LECTURE == HIGH) {
    Serial.println("Lecture fichier pas ok");
    Serial.print("Dossier = "); Serial.println(DOSSIER_CARTE);
    Serial.print("Fichier = "); Serial.println(FICHIER_CARTE);
    switch (INDICATIF_PAYS) {
      case USA: myDFPlayer.playFolder(10, 200); break; // US international call cannot be completed
      case ROYAUME_UNI: myDFPlayer.playFolder(11, 200); break; // UK service not available
      case ESPAGNE: myDFPlayer.playFolder(13, 200); break; // Espagne numéro pas en service
      default : myDFPlayer.playFolder(1, 6); break; // pas d'abonné france avec chanson
    }

  }
  else {
    Serial.println("Lecture fichier ok");

  }
  ETATEL = ENCOM;
}

void appel_reunion() {
    switch (NUMTEL){
      case 295115: DOSSIER_CARTE = 5; FICHIER_CARTE = 1; Serial.println("Appel Maliha");break; // Maliha
      default: DOSSIER_CARTE = 1; FICHIER_CARTE = 6; break;
    }
}

void appel_usa() {
    DOSSIER_CARTE = 10;

    if (AREA_CODE == 716 && NUMTEL == 8812514) {
      FICHIER_CARTE = 1; 
      Serial.println("Appel en attente");            
    }
    
    if (AREA_CODE == 229 && NUMTEL == 8521621) {
      FICHIER_CARTE = 2; 
      Serial.println("Appel Robin");            
    }

    if (AREA_CODE == 202 && NUMTEL == 7621401) {
      FICHIER_CARTE = 3; 
      Serial.println("Appel Time & weather");            
    }
}

void appel_uk() {
    DOSSIER_CARTE = 11;
    switch (NUMTEL){
      case 1123: FICHIER_CARTE = 1; Serial.println("Time"); break; // Time
      case 514735612: FICHIER_CARTE = 2; Serial.println("Rowan"); break; // Rowan
      default: FICHIER_CARTE = 200; break;
    }
}

void appel_allemagne_rfa() {
    DOSSIER_CARTE = 12;
    if (AREA_CODE == 7071 && NUMTEL == 725315) {
      FICHIER_CARTE = 1; 
      Serial.println("Appel Robin");            
    }
    else {
      FICHIER_CARTE = 200;  
    }
}

void appel_allemagne_rda() {
    DOSSIER_CARTE = 12;
    if (NUMTEL == 3312546) {
      FICHIER_CARTE = 2; 
      Serial.println("Appel Andrea");            
    }
    else {
      FICHIER_CARTE = 200;  
    }
}

void appel_espagne() {
    DOSSIER_CARTE = 13;
    switch (NUMTEL){
      case 12546235: FICHIER_CARTE = 1; Serial.println("Appel Elena"); break; // Elena
      case 44812462: FICHIER_CARTE = 2; Serial.println("Appel Magarita"); break; // Margarita
      default: FICHIER_CARTE = 200; break;
    }
}

void appel_chine() {
    DOSSIER_CARTE = 14;
    if (NUMTEL == 108765647) {
      FICHIER_CARTE = 1; 
      Serial.println("Appel Hu Zijuan");            
    }
    else {
      FICHIER_CARTE = 200;  
    }
}

void appel_taiwan() {
    DOSSIER_CARTE = 15;
    if (NUMTEL == 2863584) {
      FICHIER_CARTE = 1; 
      Serial.println("Appel Heng Yin");            
    }
    else {
      FICHIER_CARTE = 200;  
    }
}

void appel_japon() {
    DOSSIER_CARTE = 16;
    if (NUMTEL == 345261) {
      FICHIER_CARTE = 1; 
      Serial.println("Appel Ayaka-san");            
    }
    else {
      FICHIER_CARTE = 200;  
    }
}

void attendre() {
  delay(200);
}

void loop() {

// ************ LECTURE ENTREES ARDUINO ************
  ETAT_RACCROCHAGE = digitalRead(BROCHE_DECROCHAGE);
  ETAT_ROTATION = digitalRead(BROCHE_ROTATION);
  ETAT_IMPULSION = digitalRead(BROCHE_IMPULSIONS);

  // Serial.print("Etat raccrochage = "); Serial.println(ETAT_RACCROCHAGE);
  if (ETAT_RACCROCHAGE == 1) {
    ETATEL = RACCROCHE;
  }
  //      Serial.print("Etat tel = ");Serial.println(ETATEL);
  switch (ETATEL) {
    case RACCROCHE: repos(); break;
    case ATTNUM: attendre_numerotation(); break;
    case NUMEROT: numeroter(); break;
    case ACHEMIN: acheminer(); break;
    case SONNE: sonner(); break;
    case DEBCOM: etablircom(); break;
    case ENCOM:  attendre(); break;
    case ERREUR: attendre(); break;
  }


  //Serial.print("\tRotation = "); Serial.print(ETAT_ROTATION);
  //Serial.print("\tImpulsion = "); Serial.print(ETAT_IMPULSION);
  //Serial.print("\tImpulsion prec = "); Serial.print(ETAT_IMPULSION_PREC);

}
